@@ -81,8 +81,11 @@ function checkinit()
     cp(i, supportdir, testdir)
   end
   execute(os_ascii .. ">" .. testdir .. "/ascii.tcx")
+  return checkinit_hook()
 end
 
+checkinit_hook = checkinit_hook or function() return 0 end
+
 local function rewrite(source,result,processor,...)
   local file = assert(open(source,"rb"))
   local content = gsub(file:read("*all") .. "\n","\r\n","\n")
@@ -98,7 +101,7 @@ end
 -- the 'business' part from the tests and removes system-dependent stuff
 local function normalize_log(content,engine,errlevels)
   local maxprintline = maxprintline
-  if match(engine,"^lua") then
+  if match(engine,"^lua") or match(engine,"^harf") then
     maxprintline = maxprintline + 1 -- Deal with an out-by-one error
   end
   local function killcheck(line)
@@ -134,22 +137,20 @@ local function normalize_log(content,engine,errlevels)
     lastline = ""
     -- Zap ./ at begin of filename
     line = gsub(line, "%(%.%/", "(")
-    -- Zap paths if places other than 'here' are accessible
-    if checksearch then
-      -- The pattern excludes < and > as the image part can have
-      -- several entries on one line
-      local pattern = "%w?:?/[^ %<%>]*/([^/%(%)]*%.%w*)"
-      -- Files loaded from TeX: all start ( -- )
-      line = gsub(line, "%(" .. pattern, "(../%1")
-      -- Images
-      line = gsub(line, "<" .. pattern .. ">", "<../%1>")
-      -- luaotfload files start with keywords
-      line = gsub(line, "from " .. pattern .. "%(", "from. ./%1(")
-      line = gsub(line, ": " .. pattern .. "%)", ": ../%1)")
-      -- Deal with XeTeX specials
-      if match(line, "^%.+\\XeTeX.?.?.?file") then
-        line = gsub(line, pattern, "../%1")
-      end
+    -- Zap paths
+    -- The pattern excludes < and > as the image part can have
+    -- several entries on one line
+    local pattern = "%w?:?/[^ %<%>]*/([^/%(%)]*%.%w*)"
+    -- Files loaded from TeX: all start ( -- )
+    line = gsub(line, "%(" .. pattern, "(../%1")
+    -- Images
+    line = gsub(line, "<" .. pattern .. ">", "<../%1>")
+    -- luaotfload files start with keywords
+    line = gsub(line, "from " .. pattern .. "%(", "from. ./%1(")
+    line = gsub(line, ": " .. pattern .. "%)", ": ../%1)")
+    -- Deal with XeTeX specials
+    if match(line, "^%.+\\XeTeX.?.?.?file") then
+      line = gsub(line, pattern, "../%1")
     end
     -- Deal with dates
     if match(line, "[^<]%d%d%d%d[/%-]%d%d[/%-]%d%d") then
@@ -243,6 +244,9 @@ local function normalize_log(content,engine,errlevels)
        match(line, "^used structure  >") then
        return ""
     end
+    -- The first time a new font is used by LuaTeX, it shows up
+    -- as being cached: make it appear loaded every time
+    line = gsub(line, "save cache:", "load cache:")
     -- A tidy-up to keep LuaTeX and other engines in sync
     line = gsub(line, utf8_char(127), "^^?")
     -- Unicode engines display chars in the upper half of the 8-bit range:
@@ -345,9 +349,6 @@ local function normalize_lua_log(content,luatex)
         "Missing character: There is no (%^%^..) %(U%+(....)%)",
         "Missing character: There is no %1"
       )
-    -- The first time a new font is used, it shows up
-    -- as being cached
-    line = gsub(line, "(save cache:", "(load cache:")
     -- LuaTeX from v1.07 logs kerns differently ...
     -- This block only applies to the output of LuaTeX itself,
     -- hence needing a flag to skip the case of the reference log
@@ -544,9 +545,14 @@ function runcheck(name, hide)
   local function check_and_diff(ext,engine,comp,pdftest)
     runtest(name,engine,hide,ext,pdftest,true)
     local errorlevel = comp(name,engine)
-    if errorlevel ~= 0 and options["halt-on-error"] then
+    if errorlevel == 0 then
+      return errorlevel
+    end
+    if options["show-log-on-error"] then
+      showfailedlog(name)
+    end
+    if options["halt-on-error"] then
       showfaileddiff()
-      return 1
     end
     return errorlevel
   end
@@ -632,7 +638,7 @@ function compare_tlg(name,engine,cleanup)
   end
   -- Do additional log formatting if the engine is LuaTeX, there is no
   -- LuaTeX-specific .tlg file and the default engine is not LuaTeX
-  if match(engine,"^lua")
+  if (match(engine,"^lua") or match(engine,"^harf"))
     and not match(tlgfile, "%.luatex" .. "%" .. tlgext)
     and not match(stdengine,"^lua")
     then
@@ -710,13 +716,23 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
   for _,filetype in pairs(dynamicfiles) do
     rm(testdir,filetype)
   end
+  -- Ensure there is no stray .log file
+  rm(testdir,name .. logext)
   local errlevels = {}
+  local localtexmf = ""
+  if texmfdir and texmfdir ~= "" and direxists(texmfdir) then
+    localtexmf = os_pathsep .. abspath(texmfdir) .. "//"
+  end
   for i = 1, checkruns do
     errlevels[i] = run(
       testdir,
       -- No use of localdir here as the files get copied to testdir:
       -- avoids any paths in the logs
-      os_setenv .. " TEXINPUTS=." .. (checksearch and os_pathsep or "")
+      os_setenv .. " TEXINPUTS=." .. localtexmf
+        .. (checksearch and os_pathsep or "")
+        .. os_concat ..
+      os_setenv .. " LUAINPUTS=." .. localtexmf
+        .. (checksearch and os_pathsep or "")
         .. os_concat ..
       -- Avoid spurious output from (u)pTeX
       os_setenv .. " GUESS_INPUT_KANJI_ENCODING=0"
@@ -728,7 +744,7 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
       -- Ensure lines are of a known length
       os_setenv .. " max_print_line=" .. maxprintline
         .. os_concat ..
-      binary .. format 
+      binary .. format
         .. " " .. asciiopt .. " " .. checkopts
         .. setup(lvtfile)
         .. (hide and (" > " .. os_null) or "")
@@ -767,7 +783,11 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
            ext ~= tlgext and
            ext ~= lveext and
            ext ~= logext then
-           ren(testdir, file, gsub(file, "(%.[^.]+)$", "." .. engine .. "%1"))
+           local newname = gsub(file,"(%.[^.]+)$","." .. engine .. "%1")
+           if fileexists(testdir,newname) then
+             rm(testdir,newname)
+           end
+           ren(testdir,file,newname)
         end
       end
     end
@@ -907,6 +927,20 @@ function checkdiff()
   print("")
 end
 
+function showfailedlog(name)
+  print("\nCheck failed with log file")
+  for _,i in ipairs(filelist(testdir, name..".log")) do
+    print("  - " .. testdir .. "/" .. i)
+    print("")
+    local f = open(testdir .. "/" .. i,"r")
+    local content = f:read("*all")
+    close(f)
+    print("-----------------------------------------------------------------------------------")
+    print(content)
+    print("-----------------------------------------------------------------------------------")
+  end
+end
+
 function showfaileddiff()
   print("\nCheck failed with difference file")
   for _,i in ipairs(filelist(testdir, "*" .. os_diffext)) do
@@ -927,7 +961,7 @@ function save(names)
   for _,name in pairs(names) do
     if testexists(name) then
       for _,engine in pairs(engines) do
-        local testengine = ((engine == stdengine and "") or "." .. engine)        
+        local testengine = ((engine == stdengine and "") or "." .. engine)
         local function save_test(test_ext,gen_ext,out_ext,pdfmode)
           local out_file = name .. testengine .. out_ext
           local gen_file = name .. "." .. engine .. gen_ext
@@ -956,7 +990,7 @@ function save(names)
       return 1
     else
       print('Test "'.. name .. '"not found')
-      return 1     
+      return 1
     end
   end
   return 0
