@@ -1,6 +1,6 @@
 --[[
 
-File l3build-variables.lua Copyright (C) 2018 The LaTeX3 Project
+File l3build-variables.lua Copyright (C) 2018,2019 The LaTeX3 Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
@@ -46,6 +46,7 @@ sourcefiledir = sourcefiledir or currentdir
 supportdir    = supportdir    or maindir .. "/support"
 testfiledir   = testfiledir   or currentdir .. "/testfiles"
 testsuppdir   = testsuppdir   or testfiledir .. "/support"
+texmfdir      = texmfdir      or maindir .. "/texmf"
 
 -- Structure within a development area
 builddir   = builddir   or maindir .. "/build"
@@ -104,7 +105,7 @@ unpackdeps  = unpackdeps  or { }
 
 -- Executable names plus following options
 typesetexe = typesetexe or "pdflatex"
-unpackexe  = unpackexe  or "tex"
+unpackexe  = unpackexe  or "pdftex"
 zipexe     = zipexe     or "zip"
 
 checkopts   = checkopts   or "-interaction=nonstopmode"
@@ -176,10 +177,13 @@ epoch        = epoch        or 1463734800
 if flatten == nil then
   flatten = true
 end
+if flattentds == nil then
+  flattentds = true
+end
 maxprintline = maxprintline or 79
 packtdszip   = packtdszip   or false
 typesetcmds  = typesetcmds  or ""
-typesetruns  = typesetruns  or 2
+typesetruns  = typesetruns  or 3
 recordstatus = recordstatus or false
 
 -- Extensions for various file types: used to abstract out stuff a bit
@@ -197,6 +201,9 @@ tpfext = tpfext or ".tpf"
 -- Manifest options
 manifestfile = manifestfile or "MANIFEST.md"
 
+-- Non-standard installation locations
+tdslocations = tdslocations or { }
+
 -- Upload settings
 curlexe  = curlexe  or "curl"
 uploadconfig = uploadconfig or {}
